@@ -24,10 +24,13 @@ import json
 from urlparse import urlparse
 from base64 import b64encode
 import netrc
+from subprocess import Popen, PIPE
+import shlex
+from datetime import datetime
 
 DEFAULT_SERVICE_URL = "https://gracedb.ligo.org/api/"
 DEFAULT_BASIC_SERVICE_URL = "https://gracedb.ligo.org/apibasic/"
-KNOWN_TEST_HOSTS = ['moe.phys.uwm.edu', 'embb-dev.ligo.caltech.ed', 'simdb.phys.uwm.edu',]
+KNOWN_TEST_HOSTS = ['moe.phys.uwm.edu', 'embb-dev.ligo.caltech.edu', 'simdb.phys.uwm.edu',]
 
 #---------------------------------------------------------------------
 # This monkey patch forces TLSv1 if the python version is 2.6.6.
@@ -64,6 +67,82 @@ def cleanListInput(list_arg):
         stringified_list = ','.join(map(str,list_arg))            
     return stringified_list
 
+# The following are used to check whether a user has tried to use
+# an expired certificate.
+
+# Parse a datetime object out of the openssl output.
+# Note that this returns a naive datetime object.
+def get_dt_from_openssl_output(s):
+    dt = None
+    err = ''
+    # Openssl spits out a string like "notAfter=Feb  6 15:17:54 2016 GMT"
+    # So we first have to split off the bit after the equal sign.
+    try:
+        date_string = s.split('=')[1].strip()
+    except:
+        err = 'Openssl output not understood.'
+        return dt, err
+
+    # Next, attempt to parse the date with strptime.
+    try:
+        dt = datetime.strptime(date_string, "%b %d %H:%M:%S %Y %Z")
+    except:
+        err = 'Could not parse time string from openssl.'
+        return dt, err
+
+    return dt, err
+
+# Given a path to a cert file, check whether it is expired.
+def is_expired(cert_file):
+    cmd = 'openssl x509 -enddate -noout -in %s' % cert_file
+    p = Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
+    out, err = p.communicate()
+
+    expired = None
+    if p.returncode == 0:
+        dt, err = get_dt_from_openssl_output(out)
+
+        if dt:
+            # Note that our naive datetime must be compared with a UTC
+            # datetime that has been rendered naive.
+            expired = dt <= datetime.utcnow().replace(tzinfo=None)
+
+    return expired, err
+
+# A utility for writing out an error message to the user and then stopping
+# execution. This seems to behave sensibly in both the interpreter and in
+# a script.
+def output_and_die(msg):
+    sys.stderr.write(msg)
+    sys.exit(1)
+
+# Given an HTTPResponse object, try to read it's content and interpret as
+# JSON--or die trying.
+def load_json_or_die(response):
+
+    # First check that the response object actually exists.
+    if not response:
+        msg = "ERROR: no response object. \n\n"
+        output_and_die(msg)
+
+    # Next, try to read the content of the response.
+    try:
+        response_content = response.read()
+    except Exception, e:
+        msg = "ERROR: problem reading response. \n\n"
+        output_and_die(msg)
+
+    # Finally, try to create a dict by decoding the response as JSON.
+    rdict = None
+    try:
+        rdict = json.loads(response_content)
+    except Exception, e:
+        msg = "ERROR: got unexpected content from the server:\n\n"
+        msg += response_content + "\n\n"
+        output_and_die(msg)
+        
+    return rdict          
+
 #-----------------------------------------------------------------
 # Exception(s)
 
@@ -151,6 +230,11 @@ class GsiRest(object):
             cred=None):
         if not cred:
             cred = findUserCredentials()
+        if not cred:
+            msg = "\nERROR: No certificate (or proxy) found. \n\n"
+            msg += "Please run ligo-proxy-init or grid-proxy-init (as appropriate) "
+            msg += "to generate one.\n\n"
+            output_and_die(msg)
         if isinstance(cred, (list, tuple)):
             self.cert, self.key = cred
         elif cred:
@@ -169,7 +253,13 @@ class GsiRest(object):
             # Use the new method with SSL Context
             # Prepare SSL context
             ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
-            ssl_context.load_cert_chain(self.cert, self.key)
+            try:
+                ssl_context.load_cert_chain(self.cert, self.key)
+            except ssl.SSLError, e:
+                msg = "\nERROR: Unable to load cert/key pair. \n\n"
+                msg += "Please run ligo-proxy-init or grid-proxy-init again "
+                msg += "or make sure your robot certificate is readable.\n\n"
+                output_and_die(msg)
             # Generally speaking, test boxes use cheap/free certs from the LIGO CA.
             # These cannot be verified by the client.
             if host in KNOWN_TEST_HOSTS:
@@ -197,6 +287,49 @@ class GsiRest(object):
     def getConnection(self):
         return self.connector()
 
+    # When there is a problem with the SSL connection or cert authentication,
+    # either conn.request() or conn.getresponse() will throw an exception.
+    # The following two wrappers are intended to catch these exceptions and 
+    # return an intelligible error message to the user.
+    # A wrapper for getting the response:
+    def get_response(self, conn):
+        try:
+            return conn.getresponse()
+        except ssl.SSLError, e:
+            # Check for a valid user proxy cert.
+            expired, error = is_expired(self.cert)
+            if expired is not None:
+                if expired:
+                    msg = "\nERROR: Your certificate or proxy has expired. \n\n"
+                    msg += "Please run ligo-proxy-init or grid-proxy-init (as appropriate) "
+                    msg += "to generate a fresh one.\n\n"
+                else:
+                    msg = "\nERROR \n\n"
+                    msg += "Your certificate appears valid, but there was a problem "
+                    msg += "establishing a secure connection: \n\n"
+                    msg += "%s \n\n" % str(e)
+            else:
+                msg = "\nERROR \n\n"
+                msg += "Unable to check certificate expiry date: %s \n\n" %error
+                msg += "Problem establishing secure connection: %s \n\n" % str(e)
+            output_and_die(msg)
+        except Exception, e:
+            # Some error unrelated to the SSL connection has occurred.
+            output_and_die("ERROR: %s\n" % str(e))
+
+    # A wrapper for making the request.
+    def make_request(self, conn, *args, **kwargs):
+        try:
+            conn.request(*args, **kwargs)
+        except ssl.SSLError, e:
+            msg = "\nERROR \n\n"
+            msg += "Problem establishing secure connection: %s \n\n" % str(e)
+            output_and_die(msg)
+        except Exception, e:
+            msg = "\nERROR \n\n"
+            msg += "%s \n\n" % str(e)
+            output_and_die(msg)
+        
     def request(self, method, url, body=None, headers=None, priming_url=None):
         # Bug in Python (versions < 2.7.1 (?))
         # http://bugs.python.org/issue11898
@@ -211,15 +344,15 @@ class GsiRest(object):
 
         conn = self.getConnection()
         if priming_url:
-            conn.request("GET", priming_url, headers={'connection' : 'keep-alive'})
-            response = conn.getresponse()
+            self.make_request(conn, "GET", priming_url, headers={'connection' : 'keep-alive'})
+            response = self.get_response(conn)
             if response.status != 200:
                 response = self.adjustResponse(response)
             else:
                 # Throw away the response and make sure to read the body.
                 response = response.read()
-        conn.request(method, url, body, headers or {})
-        response = conn.getresponse()
+        self.make_request(conn, method, url, body, headers or {})
+        response = self.get_response(conn)
         return self.adjustResponse(response)
 
     def adjustResponse(self, response):
@@ -234,7 +367,7 @@ class GsiRest(object):
                 except:
                     pass  
             raise HTTPError(response.status, response.reason, response_content)
-        response.json = lambda: json.loads(response.read())
+        response.json = lambda: load_json_or_die(response)
         return response
 
     def get(self, url, headers=None):
@@ -305,7 +438,8 @@ class GraceDb(GsiRest):
     @property
     def service_info(self):
         if not self._service_info:
-            self._service_info = self.request("GET", self.service_url).json()
+            r = self.request("GET", self.service_url)
+            self._service_info = r.json()
         return self._service_info
 
     @property
@@ -976,9 +1110,9 @@ class GraceDbBasic(GraceDb):
                 pass 
 
         if not username or not password:
-            msg = "Could not find user credentials. " 
-            msg +="Please use a .netrc file or provide username and password." 
-            raise ValueError(msg)
+            msg = "\nERROR: Could not find user credentials. \n\n"
+            msg +="Please use a .netrc file or provide username and password.\n\n" 
+            output_and_die(msg)
 
         # Construct authorization header
         userAndPass = b64encode(b"%s:%s" % (username, password)).decode("ascii")
@@ -1015,6 +1149,33 @@ class GraceDbBasic(GraceDb):
         self.service_url = service_url
         self._service_info = None
 
+    # When there is a problem with the SSL connection or authentication,
+    # either conn.request() or conn.getresponse() will throw an exception.
+    def get_response(self, conn):
+        try:
+            return conn.getresponse()
+        except ssl.SSLError, e:
+            msg = "\nERROR \n\n"
+            msg += "Problem establishing secure connection: %s \n\n" % str(e)
+            output_and_die(msg)
+        except Exception, e:
+            msg = "\nERROR \n\n"
+            msg += "%s \n\n" % str(e)
+            output_and_die(msg)
+
+    # A wrapper for making the request.
+    def make_request(self, conn, *args, **kwargs):
+        try:
+            conn.request(*args, **kwargs)
+        except ssl.SSLError, e:
+            msg = "\nERROR \n\n"
+            msg += "Problem establishing secure connection: %s \n\n" % str(e)
+            output_and_die(msg)
+        except Exception, e:
+            msg = "\nERROR \n\n"
+            msg += "%s \n\n" % str(e)
+            output_and_die(msg)
+
     def request(self, method, url, body=None, headers=None):
         # Bug in Python (versions < 2.7.1 (?))
         # http://bugs.python.org/issue11898
@@ -1028,11 +1189,21 @@ class GraceDbBasic(GraceDb):
         conn = self.getConnection()
         headers = headers or {}
         headers.update(self.authn_header)
-        conn.request(method, url, body, headers)
-        response = conn.getresponse()
+        self.make_request(conn, method, url, body, headers)
+        response = self.get_response(conn)
+        # Catch the 401 unauthorized response before sending to adjust
+        # response. Effectively, the 401 response will have special status.
+        if response.status == 401:
+            try:
+                msg = "\nERROR: %s \n\n" % json.loads(response.read())['error']
+            except:
+                msg = "\nERROR: \n\n" 
+            msg += "Please check the username/password in your .netrc file. \n"
+            msg += "Note: If your password is more than a year old, you will \n"
+            msg += "need to use the web interface to generate a new one. \n\n" 
+            output_and_die(msg)
         return self.adjustResponse(response)
 
-
 #-----------------------------------------------------------------
 # HTTP upload encoding
 # Taken from http://code.activestate.com/recipes/146306/
@@ -1099,4 +1270,3 @@ def findUserCredentials(warnOnOldProxy=1):
     if os.path.exists(certFile) and os.path.exists(keyFile):
         return certFile, keyFile
 
-
